import common
import socket
import random

class GameServer(object):

    def __init__(self, player_count, name='', port=23456):
        self.server_name = name
        self.port = port
        self.player_count = player_count
        self.state = common.GameState(player_count)
        self.player_sockets = []

    def wait_for_players(self):
        player = 0
        s = socket.socket(socket.AF_INET,
                          socket.SOCK_STREAM)
        s.bind((self.server_name, self.port))
        s.listen(5)
        self.server_socket = s
        while player < self.player_count:
            (sock, addr) = s.accept()
            player += 1
            cs = common.CommSocket(sock)
            self.player_sockets.append(cs)

    def ask_for_player_info(self, player_socket, i):
        player_socket.send_object(common.REQUEST_INFO)
        n = player_socket.recv_object()
        return n

    def done(self):
        for player in self.player_sockets:
            player.close()
        self.server_socket.close()

    def play_game(self, randomize_order):

        def shuffle_list(lst, p):
            new_lst = []
            for i in p:
                new_lst.append(lst[i])
                del lst[i]
            lst[:] = new_lst

        def randomize_players():
            p = [random.randint(0, mx) for
                 mx in
                 reversed(xrange(len(self.player_sockets)))]
            shuffle_list(self.player_sockets, p)
            shuffle_list(self.state.known_state.players, p)
            
        # Starting sequence: First get player names
        for player, i in zip(self.player_sockets,
                             xrange(len(self.player_sockets))):
            print "Asking for player info"
            player_state = self.state.known_state.players[i]
            player_state.name = self.ask_for_player_info(player, i)
            self.player_info_received(i, player_state)

        if randomize_order:
            randomize_players()

        players_to_remove = []

        # Now, broadcast starting state
        for player, i in zip(self.player_sockets,
                             xrange(len(self.player_sockets))):
            pname = self.state.known_state.players[i].name
            print "Sending starting state to player %s" % pname
            try:
                player.send_object(self.state.known_state)
            except RuntimeError:
                self.player_is_broken(self.state.known_state.players[i])
                print "Player %s is broken. Kicking him out." % pname
                players_to_remove.append(i)

        for i in reversed(players_to_remove):
            self.player_sockets[i].close()
            del self.player_sockets[i]
            self.state.known_state.players[i]

        self.state.flip_card()

        try:
            while 1:
                current_index = self.state.known_state.current_player
                current_player = self.state.known_state.players[current_index]
                current_psocket = self.player_sockets[current_index]
                pname = current_player.name
                print "Requesting move from player %s" % pname
                try:
                    current_psocket.send_object(common.REQUEST_PLAY)
                    current_psocket.send_object(self.state.known_state)
                    response = current_psocket.recv_object()
                    if response == common.RESPONSE_TAKE:
                        print "Player #%s TOOK" % (current_index + 1)
                        self.state.take_card()
                        self.state.flip_card()
                    elif response == common.RESPONSE_PASS:
                        self.state.skip_card()
                except RuntimeError:
                    self.player_is_broken(current_player)
                    print "Invalid response. Will take card."
                    self.state.take_card()
                    self.state.flip_card()
        except common.GameOver:
            self.game_over()
            for player, i in zip(self.player_sockets,
                                 xrange(len(self.player_sockets))):
                try:
                    player.send_object(common.NOTIFY_GAMEOVER)
                    player.send_object(self.state.known_state)
                    r = player.recv_object()
                    if r != common.ACK_GAMEOVER:
                        raise RuntimeError
                except RuntimeError:
                    self.player_is_broken(self.state.known_state.players[i])

    def run_game(self, randomize_order=True):
        self.wait_for_players()
        self.play_game(randomize_order)
        self.done()

    ##########################################################################

    def client_connected(self, socket):
        pass
    
